<?

/*

sh4re - a secure peer to peer file sharing system
=================================================

Copyright (C) 2003 Vincent Negrier aka. sIX <six@aegis-corp.org>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

*/

/* 

todo :

- 0.9.30 : status handled in sh4re-cli

- 0.9.50 : S2P_PKT_PAUSE
		   S2P_PKT_RESUME
           S2P_PKT_CANCEL
		   S2P_PKT_KILL_SESSION

- 0.9.90 : sh4re-www

*/

define("SH4_VERSION", "0.9.24c");

// sharing defines

define("SH4_SEARCH_LIMIT", 100);
define("SH4_XFER_BLOCK_SIZE", 524288);

// s2p defines

define("S2P_VERSION", 1);

define("S2P_HANDLER_CLASS", "s2p_connection");
define("S2P_EH_LEN", 6);
define("S2P_H_LEN", 9);
define("S2P_B_LEN", 4);
define("S2P_MAX_PKT_SIZE", 1048576);

// S2P packet types

define("S2P_PKT_CERT", 0);
define("S2P_PKT_CHALLENGE", 1);
define("S2P_PKT_HELLO", 2);

define("S2P_PKT_MASTER_NOTICE", 10);
define("S2P_PKT_MASTER_FATAL", 11);

define("S2P_PKT_NOTICE", 15);

define("S2P_PKT_PUBMSG", 20);
define("S2P_PKT_PRIVMSG", 21);

define("S2P_PKT_INFO_NET", 30);
define("S2P_PKT_INFO_SRV", 31);
define("S2P_PKT_STATUS", 32);

define("S2P_PKT_RIGHT_LIST", 40);
define("S2P_PKT_RIGHT_SET", 41);

define("S2P_PKT_FIND", 50);

define("S2P_PKT_ASK_HASH", 60);

define("S2P_PKT_CONNECT", 100);
define("S2P_PKT_DISCONNECT", 101);

define("S2P_PKT_GET_HASHES", 110);

define("S2P_PKT_SHUTDOWN", 255);

// S2P INFO_NET packet flags

define("SH4_NI_NETWORK", 1);
define("SH4_NI_ACTIVE_NODES", 2);
define("SH4_NI_NODES", 4);
define("SH4_NI_NODE_UPDATE", 8);
define("SH4_NI_DISCONNECT", 16);
define("SH4_NI_ALL", SH4_NI_NETWORK | SH4_NI_ACTIVE_NODES | SH4_NI_NODES);

// S2P cryptography

define("S2P_NETKEY_CIPHER", MCRYPT_TWOFISH);
define("S2P_CRYPT_BLOCKSIZE", 96);

// S2P fake response to bad packets

$S2P_BAD_RESPONSE = 'HTTP/1.0 404 Not Found
Server: Microsoft-IIS/5.0
Content-Type: text/html
Connection: close

<!DOCTYPE HTML PUBLIC "-//IETF//DTD HTML 2.0//EN">
<html><head>
<title>404 Not Found</title>
</head><body>
<h1>Not Found</h1>
<p>The requested URL was not found on this server.</p>
<hr>
</body></html>
';

// shadok netlib

define("SH4_SHADOK_VERSION", "shadok-0.8.2a");
include SH4_INC_DIR.DIRECTORY_SEPARATOR.SH4_SHADOK_VERSION.DIRECTORY_SEPARATOR."shadok.php";

set_time_limit(0);

// S2P base handler

class s2p_connection extends SK_Connection_Handler {

	var $handshake = 0;
	var $s2p_challenge = "";
	var $s2p_status = 0;
	var $s2p_buffer = "";
	var $s2p_errors = array();
	var $remote_cert_s = "";
	var $remote_cert = false;
	var $remote_key = false;
	var $remote_ip;
	var $remote_port;
	var $remote_s2p_agent = "";
	var $last_read_time = 0;

	var $ctype;
	var $packet = array();
	var $options = array();
	
	/* $options array :

	s2p_agent			: s2p_agent string
	net_key				: symetric encryption key (string)
	use_rsa				: use asymetric encryption (bool)
		my_key				: asymetric private encryption key (resource)
		my_cert_s			: certificate to send (string)	
		check_client_cert  : match sent cert against network database (masterd-only) (bool)
			my_ca				: certificate authority (whatever openssl_x509_checkpurpose accepts)
		check_network_cert  : match sent cert against stored network cert (bool)
			new_nework_cert		: accept a new certificate and store it as the network cert (bool)


	*/
	
	function on_accept() {

		$this->ctype = "server";		
		$this->handshake = 0;

		list($this->remote_ip, $this->remote_port)=$this->socket->getpeername();

	}

	function on_connect() {

		$this->ctype = "client";
		$this->handshake = 0;

		if ($this->options["s2p_agent"]) {
			
			$hpkt = $this->options["s2p_agent"];

		} else {

			$hpkt = "sh4re-core/".SH4_VERSION;
		
		}
		
		$this->s2p_write(S2P_PKT_HELLO, $hpkt, false);

	}
	
	function on_read($data) {

		$this->last_read_time = time();
		$this->s2p_buffer .= $data;
		
		while (true) {
		
			if (($this->s2p_status == 0) && (strlen($this->s2p_buffer) >= S2P_EH_LEN)) {

				$eh_packed = substr($this->s2p_buffer, 0, S2P_EH_LEN);
				$this->s2p_buffer = substr($this->s2p_buffer, S2P_EH_LEN);

				$this->packet = unpack("Cversion/Cencrypted/Vs2p_layer_length", $eh_packed);

				if ($this->packet["version"] != S2P_VERSION) {
					
					$this->s2p_errors[] = "incompatible s2p protocol version";
					$this->fake_disconnect();

				}
				
				$this->s2p_status = 1;

			} else if (($this->s2p_status == 1) && (strlen($this->s2p_buffer) >= $this->packet["s2p_layer_length"])) {

				$s2p_packet_raw = substr($this->s2p_buffer, 0, $this->packet["s2p_layer_length"]);
				$this->s2p_buffer = substr($this->s2p_buffer, $this->packet["s2p_layer_length"]);

				if ($this->packet["s2p_layer_length"]>(2*S2P_MAX_PKT_SIZE)) {
					
					$this->s2p_errors[] = "s2p packet too long";
					$this->fake_disconnect();
				
				}
				
				if ($this->options["net_key"]) {

					$s2p_packet_raw = @mcrypt_decrypt(S2P_NETKEY_CIPHER, $this->options["net_key"], $s2p_packet_raw, MCRYPT_MODE_CFB);

				}
				
				if ($this->packet["encrypted"]) {

					$s2p_packet = "";
					
					while ($s2p_packet_block_header = substr($s2p_packet_raw, 0, S2P_B_LEN)) {

						$s2p_bhdr = unpack("Vlength", $s2p_packet_block_header);

						if (($s2p_block_length = $s2p_bhdr["length"]) == 0) break;

						$s2p_packet_block_data = substr($s2p_packet_raw, S2P_B_LEN, $s2p_block_length);
						openssl_private_decrypt($s2p_packet_block_data, $s2p_packet_block, $this->options["my_key"]);
						$s2p_packet = $s2p_packet_block.$s2p_packet;

						$s2p_packet_raw = substr($s2p_packet_raw, $s2p_block_length + S2P_B_LEN);

					}
				
				} else {

					$s2p_packet = $s2p_packet_raw;

				}

				$s2p_header = substr($s2p_packet, 0, S2P_H_LEN);
				$s2p_load = substr($s2p_packet, S2P_H_LEN);

				if (!$s2p_h_arr = @unpack("Ctype/Vlength/Vcrc32", $s2p_header)) {
					
					$this->s2p_errors[] = "invalid s2p packet received (corrupt header)";
					$this->fake_disconnect();

				}

				if (strlen($s2p_load) != $s2p_h_arr["length"]) {
					
					$this->s2p_errors[] = "invalid s2p packet received (packet length)";
					$this->fake_disconnect();

				}

				if (crc32($s2p_load) != $s2p_h_arr["crc32"]) {
					
					$this->s2p_errors[] = "invalid s2p packet received (packet crc32)";
					$this->fake_disconnect();

				}

				if ($s2p_h_arr["type"] == S2P_PKT_MASTER_FATAL) $this->handshake = 1;
				
				if ($this->handshake == 0) {
				
					// handshake phase
					
					if ($s2p_h_arr["type"] == S2P_PKT_CERT) {
					
						if ($this->ctype == "client") {

							// client : got server cert !

							$this->remote_cert_s = $s2p_load;
							$this->remote_cert = openssl_x509_read($this->remote_cert_s);
							$this->remote_key = openssl_pkey_get_public($this->remote_cert_s);

							if ($this->options["check_network_cert"]) {
							
								$cert_info = openssl_x509_parse($this->remote_cert);
								
								$this->remote_cert_cn = $cert_info["subject"]["CN"];
								
								if ($t_cert_l = @file_get_contents($this->options["network_cert_file"])) {
									
									$t_cert = openssl_x509_read($t_cert_l);
									openssl_x509_export($t_cert, $t_cert_s);

								}

								openssl_x509_export($this->remote_cert, $n_cert_s);

								if ($t_cert_s != $n_cert_s) {

									if ($this->options["new_network_cert"]) {

										if (!$f = fopen($this->options["network_cert_file"], "w")) errexit("FATAL: unable to write to ".$this->options["network_cert_file"]);
										fwrite($f, $n_cert_s);
										fclose($f);

										techo("network master server SSL certificate registered");
									
									} else {

										errexit("FATAL: unknown master server certificate, use --new-network-cert");
									
									}
								
								}

							}
							
							$this->s2p_write(S2P_PKT_CERT, $this->options["my_cert_s"]);
							
						} else if ($this->ctype == "server") {

							// server : got client cert !

							$this->remote_cert_s = $s2p_load;
							$this->remote_cert = openssl_x509_read($this->remote_cert_s);
							$this->remote_key = openssl_pkey_get_public($this->remote_cert_s);

							if ($this->options["check_client_cert"]) {
								
								$cert_info = openssl_x509_parse($this->remote_cert);
								$cert_signed = openssl_x509_checkpurpose($this->remote_cert, X509_PURPOSE_ANY, $this->options["my_ca"]);

								$cert_cn = $cert_info["subject"]["CN"];
								
								if (!$cert_signed) {

									$errmsg = "client certificate for '$cert_cn' is not issued by my CA, aborting connection";

									$this->s2p_write(S2P_PKT_MASTER_FATAL, $errmsg, false, true);
									$this->s2p_errors[] = $errmsg;
									
									return;
								
								}

								if (!$remote_node = sh4_get_node_by_name($cert_cn)) {

									$errmsg = "client '$cert_cn' is not in my database, aborting connection";

									$this->s2p_write(S2P_PKT_MASTER_FATAL, $errmsg, false, true);
									$this->s2p_errors[] = $errmsg;

									return;
								
								}

								$this->remote_node = $remote_node;

							}
							
							$this->s2p_challenge = md5((string)getmyinode().(string)getmypid().(string)time());
							$this->s2p_write(S2P_PKT_CHALLENGE, $this->s2p_challenge);

						} 
				
					} else if ($s2p_h_arr["type"] == S2P_PKT_CHALLENGE) {

						// post auth challenge
						
						if ($this->ctype == "client") {

							// client : got challenge !

							$this->s2p_write(S2P_PKT_CHALLENGE, md5($s2p_load));
						
						} else if ($this->ctype == "server") {

							// server : got response !

							if ($s2p_load == md5($this->s2p_challenge)) {

								if ($this->options["s2p_agent"]) {
									
									$hpkt = $this->options["s2p_agent"];

								} else {

									$hpkt = "sh4re-core/".SH4_VERSION;
								
								}
								
								$this->s2p_write(S2P_PKT_HELLO, $hpkt);
								
								$this->handshake = 1;
								$this->on_s2p_accept();
							
							} else {

								$errmsg = "failed s2p handshake challenge";

								$this->s2p_write(S2P_PKT_MASTER_FATAL, $errmsg, true, true);
								$this->s2p_errors[] = $errmsg;

								return;

							}

						}
						
					} else if ($s2p_h_arr["type"] == S2P_PKT_HELLO) {

						$this->remote_s2p_agent = $s2p_load;
						
						if ($this->ctype == "client") {

							// client : connected !

							$this->handshake = 1;
							$this->on_s2p_connect();

						} else if ($this->ctype == "server") {

							// server : client wants to connect !
							
							if ($this->options["use_rsa"]) {

								$this->s2p_write(S2P_PKT_CERT, $this->options["my_cert_s"], false);

							} else {

								$this->s2p_challenge = md5((string)getmyinode().(string)getmypid().(string)time());
								$this->s2p_write(S2P_PKT_CHALLENGE, $this->s2p_challenge);
							
							}
						
						}

					}

				} else {
				
					// forward packet to user
					
					$this->on_s2p_packet($s2p_h_arr["type"], $s2p_load);

				}

				$this->s2p_status = 0;
			
			} else {

				// no more data waiting to be processed

				break;
			
			}

		}
	
	}

	function on_s2p_accept() {

	}

	function on_s2p_connect() {

	}
	
	function on_s2p_packet($packet_type, $packet_load) {

	}

	function on_s2p_fatal($error_msg) {

	}
	
	function s2p_write($packet_type, $packet_load="", $encrypt=true, $exit=false) {

		$header = pack("CVV", $packet_type, strlen($packet_load), crc32($packet_load));
		$s2p_packet = $header.$packet_load;

		if ($encrypt && $this->options["use_rsa"] && $this->remote_key) {

			$s2p_raw = "";
			
			while ($s2p_packet) {

				$s2p_block = substr($s2p_packet, 0, S2P_CRYPT_BLOCKSIZE);
				openssl_public_encrypt($s2p_block, $s2p_block_raw, $this->remote_key);
				$s2p_raw = pack("V", strlen($s2p_block_raw)).$s2p_block_raw.$s2p_raw;

				$s2p_packet = substr($s2p_packet, S2P_CRYPT_BLOCKSIZE);

			}

			$s2p_raw .= pack("V", 0);

		} else {

			$s2p_raw = $s2p_packet;
		
		}

		if ($this->options["net_key"]) {

			$s2p_raw = @mcrypt_encrypt(S2P_NETKEY_CIPHER, $this->options["net_key"], $s2p_raw, MCRYPT_MODE_CFB);
		
		}
		
		$eh = pack("CCV", S2P_VERSION, ($encrypt && $this->options["use_rsa"])?1:0, strlen($s2p_raw));

		$packet = $eh.$s2p_raw;

		/* evil debug code
		
		$f = fopen("/tmp/s2p-".$this->ctype, "a");
		fwrite($f, $packet);
		fclose($f);

		*/

		if (!$exit) {
		
			$this->socket->buffered_write($packet);

		} else {

			$this->socket->buffered_write($packet, $this, "disconnect");
		
		}

	}

	function s2p_broadcast($packet_type, $packet_load="", $encrypt=true, $exit=false) {

		foreach ($this->server->active_sessions as $as) if ($as->handshake) $as->s2p_write($packet_type, $packet_load, $encrypt, $exit);
	
	}

	function fake_disconnect() {

		$this->on_s2p_fatal(implode(", ", $this->s2p_errors));
		$this->socket->buffered_write($GLOBALS["S2P_BAD_RESPONSE"], $this, "disconnect");
	
	}

}

// DB funcs

function sh4_get_node_by_id($node_id) {

	$q = mysql_query("select * from nodes where node_id=".(int)$node_id);
	$r = mysql_fetch_assoc($q);
	mysql_free_result($q);

	return $r;

}

function sh4_get_node_by_name($node_name) {

	$q = mysql_query("select * from nodes where node_short_name=\"".addslashes($node_name)."\"");
	$r = mysql_fetch_assoc($q);
	mysql_free_result($q);

	return $r;

}

function sh4_put_node($node) {

	$q = mysql_query("select 1 from nodes where node_id=".(int)$node["node_id"]);
	$n = mysql_fetch_row($q);
	mysql_free_result($q);

	if (!$n) return false;

	$sql = "update nodes set node_id=".$node["node_id"].", node_name=\"".addslashes($node["node_name"])."\", node_url=\"".addslashes($node["node_url"])."\", node_motd=\"".addslashes($node["node_motd"])."\", node_files_a=".$node["node_files_a"].", node_files_v=".$node["node_files_v"].", node_files_o=".$node["node_files_o"].", node_mbytes_a=".$node["node_mbytes_a"].", node_mbytes_v=".$node["node_mbytes_v"].", node_mbytes_o=".$node["node_mbytes_o"]." where node_id=".$node["node_id"];
	mysql_query($sql);

	return true;

}

function sh4_find_files($search_str, $sql_limit=SH4_SEARCH_LIMIT) {

	/* search_str syntax examples :

	- goa type=audio
	- 'star wars' type=video size>500M
	- 'amon tobin'
	- age<5d
	- samedir=d32326d5a9037678ab14cb39b5e6e84d

	*/
	
	if (!trim($search_str)) return false;
	
	$str_len = strlen($search_str);

	$arg = "";
	$args = array();
	
	for ($a=0; $a<$str_len; $a++) {

		$c = $search_str{$a};

		switch ($c) {
			
			case "\"";
			case "'";
			
			if ($c === $inside_quotes) {

				$inside_quotes = false;
			
			} else if (!$inside_quotes) {

				$inside_quotes = $c;
							
			} else {

				$arg .= $c;
			
			}

			break;


			case " ":
			
			if (!$inside_quotes) {

				$args[] = $arg;
				$arg = "";
			
			} else {

				$arg .= $c;
			
			}
			
			break;
			
			
			default:

			if (($inside_quotes) && (in_array($c, array("<", "=", ">")))) $c = "\\".$c;
			$arg .= $c;
			
			break;
		
		}

	}

	if ($arg) $args[] = $arg;

	if (!$args) return false;
	
	$operators = array("<=", ">=", ">", "<", "=");
	$args_a = array();
	
	foreach ($args as $arg) {

		$of = false;
		
		foreach ($operators as $op) if (($p = strpos($arg, $op)) && ($arg{$p-1} != '\\')) {

			$of = true;

			$arg_k = strtoupper(substr($arg, 0, $p));
			$arg_v = substr($arg, $p+1);
			
			break;

		}

		if (!$of) {

			$arg_k = "TEXT";
			$arg_v = $arg;

		}

		if ($arg_k != "TEXT") {
		
			$args_a[$arg_k] = array("op" => $op, "val" => $arg_v);

		} else {

			$args_a["TEXT"][] = $arg_v;
		
		}
	
	}
	
	$sql_tables = array("files");
	$sql_where = array();
	$text_s = array();
	$meta_search = true;

	foreach ($args_a as $type=>$query) switch ($type) {

		case "AGE":

		$u = substr($query["val"], -1);
		$v = substr($query["val"], 0 , -1);

		switch (strtoupper($u)) {

			case "Y":
			$age = (int)$v*86400*365;
			break;

			case "M":
			$age = (int)$v*86400*31;
			break;

			case "W":
			$age = (int)$v*86400*8;
			break;
			
			case "D":
			default:
			if (strtoupper($u) == "D") $query["val"] = (int)substr($query["val"], 0, -1);
			if (is_int($query["val"])) $age = $query["val"]*86400;
			else return false;
			break;
		
		}
		
		$sql_where[] = "date_sub(now(), interval $age second) ".$query["op"]." files.file_date";
		
		break;

		
		case "TYPE":

		switch (strtoupper($query["val"]{0})) {

			case "A":
			$sql_where[] = "files.file_type = \"Audio\"";
			break;

			case "V":
			$sql_where[] = "files.file_type = \"Video\"";
			break;

			case "O":
			$sql_where[] = "files.file_type = \"Other\"";
			break;

			default:
			return false;
			break;
		
		}

		break;

		
		case "SIZE":
		
		$u = substr($query["val"], -1);
		$v = substr($query["val"], 0 , -1);

		switch (strtoupper($u)) {

			case "G":
			$size = (int)$v*1024*1024*1024;
			break;

			case "M":
			$size = (int)$v*1024*1024;
			break;

			case "K":
			$size = (int)$v*1024;
			break;

			default:
			if (is_int($query["val"])) $size = $query["val"];
			else return false;
			break;
		
		}
		
		$sql_where[] = "files.file_size ".$query["op"]." ".$size;

		break;


		case "TEXT":
		foreach ($query as $s) $text_s[] = $s;
		break;

		case "META":
		if ((!$query["val"]) || (strtoupper($query["val"]) == "OFF") || (strtoupper($query["val"]{0}) == "N")) $meta_search = false;
		break;

		case "SAMEDIR":
		if ($query["op"] != "=") return false;
		if (!$sfile_id = sh4_get_file_id_by_hash($query["val"])) return false;
		$sfile = sh4_get_file($sfile_id, false);
		$sdir = $sfile["file_directory"];
		$sql_where[] = "files.file_directory = \"".addslashes($sdir)."\"";
		break;
		
		default:
		return false;
		break;
	
	}

	$text_where = array();
	
	if ($text_s) foreach ($text_s as $text) {

		$sql_str = "\"%".str_replace(" ", "%", addslashes($text))."%\"";
		
		$text_where[] = "files.file_parent_dir like $sql_str";
		$text_where[] = "files.file_name like $sql_str";
		
		if ($meta_search) {

			if (!in_array("files_metadata", $sql_tables)) {
				
				$sql_tables[] = "files_metadata";
				$sql_where[] = "files.file_id = files_metadata.meta_file_id";

			}

			$text_where[] = "files_metadata.meta_value like $sql_str";
		
		}
	
	}

	if ($text_where_s = $text_where?("(".implode(" or ", $text_where).")"):"") $sql_where[] = $text_where_s;

	$sql_tables_s = implode(", ", $sql_tables);
	$sql_where_s = implode(" and ", $sql_where);

	$sql = "select distinct files.file_id from $sql_tables_s where $sql_where_s limit $sql_limit";

	$match_ids = array();
	$q = mysql_query($sql);
	while ($r = mysql_fetch_row($q)) $match_ids[] = $r[0];
	mysql_free_result($q);

	return $match_ids;

}

function sh4_build_result($search_res, $local=false) {

	$ret = array();
	
	if ($search_res) foreach ($search_res as $k=>$file_id) {

		$f = sh4_get_file($file_id);

		unset($f["file_id"]);
		unset($f["file_directory"]);
		unset($f["file_parent_dir"]);
		if ($local) $f["avail"] = 1;
		$ret[] = $f;
	
	}

	return $ret;

}

function sh4_compute_hash($filename) {

	$fs = filesize($filename);
	
	if (!$f=@fopen($filename, "r")) return false;

	$hstr = fread($f, 150000);
	
	if ($fs>150000) {
		
		fseek($f, -150000, SEEK_END);
		$hstr .= fread($f, 150000);

	}

	$hstr .= $fs;

	$hash = md5($hstr);

	return $hash;

}

function sh4_get_file_id_by_DN($file_directory, $file_name) {

	$q = mysql_query("select file_id from files where file_directory = \"".addslashes($file_directory)."\" and file_name = \"".addslashes($file_name)."\"");
	$r = mysql_fetch_row($q);
	mysql_free_result($q);

	return $r[0];

}

function sh4_get_file_ids_by_dir($dirname) {

	$q = mysql_query("select file_id from files where file_directory=\"".addslashes($dirname)."\"");
	while ($r = mysql_fetch_row($q)) $ret[] = $r[0];
	mysql_free_result($q);

	return $ret;

}

function sh4_get_file_id_by_hash($hash) {

	$q = mysql_query("select file_id from files where file_hash=\"".addslashes($hash)."\"");
	$r = mysql_fetch_row($q);
	mysql_free_result($q);

	return $r[0];

}

function sh4_kill_file($file_id) {

	mysql_query("delete from files where file_id=".(int)$file_id);
	mysql_query("delete from files_metadata where file_id=".(int)$file_id);

}

function sh4_get_file($file_id, $with_metadata=true) {

	$q = mysql_query("select * from files where file_id=".(int)$file_id);
	$ret = mysql_fetch_assoc($q);
	mysql_free_result($q);

	if (!$ret) return false;
	
	if ($with_metadata) {
	
		$q = mysql_query("select * from files_metadata where meta_file_id=".(int)$file_id);
		while ($r = mysql_fetch_row($q)) $ret[$r[2]] = $r[3];
		mysql_free_result($q);

	}
	
	return $ret;	
	
}

function sh4_put_file($file) {

	$mainupd_cols = $mainupd_vals = $meta_sql = array();
	
	if (!$file["file_id"]) {

		mysql_query("insert into files (file_id) values (0)");
		$file["file_id"] = mysql_insert_id();

	}

	foreach ($file as $k=>$v) {

		if (in_array($k, $GLOBALS["main_cols"])) {

			$mainupd_cols[] = $k;
			$mainupd_vals[] = "\"".addslashes($v)."\"";
		
		} else {

			$meta_sql[] = "insert into files_metadata (meta_id, meta_file_id, meta_key, meta_value) values (0, ".$file["file_id"].", \"".addslashes($k)."\", \"".addslashes($v)."\")";

		}
	
	}

	$main_sql = "replace into files (".implode(", ", $mainupd_cols).") values (".implode(", ", $mainupd_vals).")";

	mysql_query($main_sql);

	if ($meta_sql) {
		
		mysql_query("delete from files_metadata where meta_file_id=".$file["file_id"]);
		
		foreach ($meta_sql as $msql) {

			mysql_query($msql);

		}

	}

	return $file["file_id"];

}


// Crypto funcs

function ssl_check_keys($key, $public) {

	$et_msg1 = md5((string)time());

	openssl_public_encrypt($et_msg1, $et_msg2, $public);
	openssl_private_decrypt($et_msg2, $et_msg3, $key);

	return ($et_msg1 == $et_msg3);

}

// Misc funcs

function sh4_serialize($mixed) {

	return serialize($mixed);

}

function sh4_unserialize($str) {

	return unserialize($str);

}

function sh4_parse_range($s, $rmax) {
	
	$ret = array();
	$rng_a = explode(",", $s);
	
	foreach ($rng_a as $rng) {

		$rng = trim($rng);
		$p = strpos($rng, "-");
		$l = strlen($rng);

		$rng_start = 0;
		$rng_end = $rmax;
		
		if ($rng === "*") {

			// "*"
		
		} else if ($p === false) {

			// "##"

			$rng_start = $rng_end = $rng;
		
		} else if ($p === 0) {

			// "-##"

			$rng_end = (int)substr($rng, 1);

		} else if ($p == ($l - 1)) {

			// "##-"

			$rng_start = $rng;

		} else {

			// "##-##";

			$rng_e = explode("-", $rng);

			$rng_start = $rng_e[0];
			$rng_end = $rng_e[1];

		}
	
		for ($a = $rng_start; $a <= $rng_end; $a++) if (!in_array($a, $ret)) $ret[] = $a;
	
	}

	return $ret;

}

function sh4_parse_cmdline_args($args_long) {

	$ret = array();
	
	for ($a=1; $a<$GLOBALS["argc"]; $a++) {

		$arg = $GLOBALS["argv"][$a];
		
		if (strpos($arg, "=")) {

			$arg = trim(strtok($arg, "="));
			$val = trim(strtok(""));
		
		} else {

			$arg = trim($arg);
			$val = true;
		
		}
	
		if ($s = array_search($arg, $args_long)) {

			$arg = $s;
		
		} 
		
		$ret[$arg] = $val;
	
	}

	return $ret;

}

function sh4_parse_config($filename) {

	if ((!@is_readable($filename)) || (!$ca = file($filename))) return false;

	$conf = array();

	foreach ($ca as $cl) {

		$cl = trim($cl);
		if ((!$cl) || ($cl{0} == "#")) continue;

		$key = strtolower(trim(strtok($cl, " \t")));
		$val = trim(strtok(""));

		$conf[$key][] = $val;
	
	}

	return $conf;

}

function techo($msg) {

	$s = date("Ymd:His")." ".$msg."\n";
	echo $s;

}

function errexit($msg) {

	techo($msg);
	exit(-1);

}

?>
