<?

/*

Shadok, the aEGiS PHP Server Daemon Toolkit
===========================================

Copyright (C) 2003 Vincent Negrier aka. sIX <six@aegis-corp.org>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

*/

define("SK_VERSION", "0.8.33a");
define("SK_INFO", "shadok/".SK_VERSION." PHP netlib (c) 2003 by sIX / aEGiS");

/* TODO

- udp (0.9)

*/

class SK_Socket {

	var $fd;

	function SK_Socket($domain=false, $type=false, $proto=false) {

		if (($domain !== false) && ($type !== false) && ($proto !== false)) $this->fd=socket_create($domain, $type, $proto);

	}
	
	function setopt($level, $optname, $optval) {

		@socket_set_option($this->fd, $level, $optname, $optval);
	
	}

	function getopt($level, $optname) {

		return (@socket_get_option($this->fd, $level, $optname));
	
	}
	
	function last_error() {

		return (@socket_last_error($this->fd));
	
	}
	
	function last_error_msg() {

		return (socket_strerror(@socket_last_error($this->fd)));
	
	}
	
	function set_nonblock() {

		@socket_set_nonblock($this->fd);
	
	}

	function connect($addr, $port) {

		return (@socket_connect($this->fd, $addr, $port));
	
	}
	
	function read($size) {

		return (@socket_read($this->fd, $size));
	
	}

	function write($msg) {

		return (@socket_write($this->fd, $msg));

	}

	function close() {

		return (@socket_close($this->fd));

	}

	function getpeername() {

		$remote_ip=$remote_port=0;

		@socket_getpeername($this->fd, $remote_ip, $remote_port);

		return array($remote_ip, $remote_port);
	
	}

}

class SK_BW_Enabled_Socket extends SK_Socket {

	var $server;
	
	function buffered_write($msg, $cb_obj=false, $cb_method=false) {
		
		$r = $this->write($msg);
		
		if ($r != strlen($msg)) {

			$ret = $this->server->New_Buffered_Write($this, substr($msg, $r), $cb_obj, $cb_method);

		} else {

			if (is_object($cb_obj)) {
			
				if (is_callable(array($cb_obj, $cb_method))) $cb_obj->$cb_method(true);
			
			}

			$ret = true;
		
		}
		
		return $ret;
	
	}

}

class SK_Listen_Socket extends SK_Socket {

	var $accepted;
	var $server;

	function listen(&$server_obj, $addr, $port, $backlog=20) {

		$this->server =& $server_obj;
		
		$ret=@socket_bind($this->fd, $addr, $port);
		if ($ret==false) return false;

		$ret=@socket_listen($this->fd, $backlog);
		if ($ret==false) return false;

		return true;

	}

	function &accept($bw_enabled=false) {

		if (is_resource($ret=socket_accept($this->fd))) {

			++$this->accepted;

			if ($bw_enabled) {

				$sck =& new SK_BW_Enabled_Socket();
				$sck->server =& $this->server;
				
			} else {
				
				$sck =& new SK_Socket();

			}

			$sck->fd=$ret;

			return $sck;

		} else {

			return false;

		}

	}

}

class SK_Server {

	var $active_listeners=array();
	var $active_sessions=array();
	var $active_writebuffers=array();

	var $last_error;

	var $options=array();

	function SK_Server() {

		// Set default server options		

		$this->options["reuse_address"]=true;
		$this->options["socket_sndbuf"]=16384;
		$this->options["socket_rcvbuf"]=8192;
		$this->options["enable_buffered_write"]=true;
		$this->options["max_atom_send"]=1048576;

	}

	function Parse_PAP($pap) {

		// Parse a "proto://addr:port" string

		$tmp=array();

		if (ereg("^([a-zA-Z]+)://([^:]+):([0-9]+)$", $pap, $tmp)) {

			$addr = $tmp[2];
			if (!ereg("^[0-9]+\.[0-9]+\.[0-9]+\.[0-9]$", $addr)) $addr = gethostbyname($addr);
			
			return array("proto" => strtolower($tmp[1]), "addr" => (($addr=="*")?"0.0.0.0":$addr), "port" => $tmp[3]);

		} else {

			return false;

		}

	}

	function New_Buffered_Write($sck, $send_buf, $callback_obj=false, $callback_method=false) {

		$this->active_writebuffers[$sck->fd][] =& new SK_Buffered_Write($sck, $send_buf, $callback_obj, $callback_method);

	}

	function Set_Option($name, $value) {

		$this->options[strtolower($name)]=$value;

	}

	function &Register_Connection_Handler($pap, $connection_handler, $listen_backlog=20, $options=false) {

		$pap=strtolower($pap);
		$p=sk_server::parse_pap($pap);

		if (is_array($this->active_listeners[$pap])) {

			$this->last_error=$pap." is already registered";
			return false;

		}

		if ($p["proto"]!="tcp") {

			$this->last_error="connection handler is only for TCP sockets";
			return false;

		}

		$s =& new sk_listen_socket(AF_INET, SOCK_STREAM, SOL_TCP);

		if (!is_resource($s->fd)) {

			$this->error="can not create socket";
			return false;

		}

		if ($this->options["reuse_address"]) $s->setopt(SOL_SOCKET, SO_REUSEADDR, 1);
		$s->set_nonblock();

		$b=$s->listen($this, $p["addr"], $p["port"], $listen_backlog);

		if ($b==false) {

			$this->error=$s->last_error_msg();
			return false;

		}

		$this->active_listeners[$pap] =& new SK_Connection_Listener($pap, $s, $connection_handler, $options);

		return $this->active_listeners[$pap];

	}

	function Unregister_Connection_Handler($pap) {

		if (!is_object($this->active_listeners[$pap])) return false;
			
		$this->active_listeners[$pap]->socket->close();
		unset($this->active_listeners[$pap]);
		
		return true;
	
	}

	function Register_Outgoing_Connection($pap, $connection_handler, $options=false) {

		$pap = strtolower($pap);
		$p = sk_server::parse_pap($pap);

		if ($p["proto"]!="tcp") {

			$this->last_error="connection handler is only for TCP sockets";
			return false;

		}

		$s =& new SK_BW_Enabled_Socket(AF_INET, SOCK_STREAM, SOL_TCP);

		$s->server =& $this;
		
		if (!is_resource($s->fd)) {

			$this->error="can not create socket";
			return false;

		}

		$b = $s->connect($p["addr"], $p["port"]);

		if ($b==false) {

			$this->error=$s->last_error_msg();
			return false;

		}

		$s->set_nonblock();
		
		$conn_id = $pap."!".(int)++$this->nb_outgoing;
		
		$hnd =& new $connection_handler;
		$hnd->conn_id = $conn_id;
		$hnd->server =& $this;
		$hnd->socket = $s;
		if ($options !== false) $hnd->options = $options;
		$this->active_sessions[$conn_id] =& $hnd;
		$this->active_sessions[$conn_id]->on_connect();

		$this->Buffered_Writes();

		return $conn_id;
	
	}
	
	function Get_All_Sockets_R() {

		$allsocks=array();

		foreach ($this->active_listeners as $al) $allsocks[]=$al->socket->fd;
		foreach ($this->active_sessions as $as) $allsocks[]=$as->socket->fd;

		return ($allsocks);

	}

	function Get_All_Sockets_W() {

		$allsocks=array();

		foreach ($this->active_writebuffers as $aw) if ($aw[0]->blocked) $allsocks[]=$aw[0]->socket->fd;

		return ($allsocks);

	}

	function Broadcast($bcast) {

		foreach ($this->active_sessions as $as) $as->socket->write($bcast);

	}

	function &Is_Active_Listener($rsock) {

		foreach ($this->active_listeners as $key=>$al) if ($al->socket->fd===$rsock) return ($this->active_listeners[$key]);
		return false;

	}

	function &Is_Active_Session($rsock) {

		foreach ($this->active_sessions as $key=>$as) if ($as->socket->fd===$rsock) return ($this->active_sessions[$key]);
		return false;

	}

	function &Is_Active_Write_Buffer($rsock) {

		foreach ($this->active_writebuffers as $key=>$aw) if ($aw->socket->fd===$rsock) return ($this->active_writebuffers[$key]);
		return false;

	}

	function Run($loops=false, $u_tm=50) {

		$nb_loops=0;

		if (($loops === 0) || ($loops === -1)) {

			if ($loops === -1) $u_timeout = $u_tm;
			$loops = 1;
			$no_timeout = true;
		
		}
		
		while (($loops===false) || (++$nb_loops<=$loops)) {

			// Incoming connections or data

			$rsocks=$this->Get_All_Sockets_R();
			$wsocks=$this->Get_All_Sockets_W();
			$dummy=NULL;

			if (@socket_select($rsocks, $wsocks, $dummy, ($no_timeout?0:1), ($u_timeout?$u_timeout:0))) {

				foreach ($rsocks as $rsock) {

					if ($al =& $this->Is_Active_Listener($rsock)) {

						$pap=$al->pap;
						$s=$al->socket->accept($this->options["enable_buffered_write"], $this);
						$conn_id=$pap."#".$this->active_listeners[$pap]->socket->accepted;
						$s->set_nonblock();

						list($remote_ip, $remote_port)=$s->getpeername();

						$hnd =& new $al->connection_handler;
						$hnd->conn_id=$conn_id;
						$hnd->server =& $this;
						$hnd->socket=$s;
						$hnd->options = $this->active_listeners[$pap]->options;
						$this->active_sessions[$conn_id] =& $hnd;
						$this->active_sessions[$conn_id]->on_accept();

					} else if ($as =& $this->Is_Active_Session($rsock)) {

						$buf=$as->socket->read(4096);

						if ($buf==="") {

							// reset by peer

							$as->on_disconnect();
							unset($this->active_sessions[$as->conn_id]);

						} else {

							$as->on_read($buf);

						}

					}

				}

				foreach ($wsocks as $wsock) if ($this->active_writebuffers[(int)$wsock]) {
					
					$this->active_writebuffers[$wsock][0]->blocked = false;
					$aw[0]->blocked=false;


				}

			}

			// Buffered socket write operations

			$this->Buffered_Writes();

		}

	}

	function Buffered_Writes() {

		foreach ($this->active_writebuffers as $key=>$wb) {

			$wss=0;
			
			if (!$wb[0]->blocked) do {
				
				$sb = $wb[0]->socket->write(substr($wb[0]->send_buffer, 0, $this->options["socket_sndbuf"]));

				$wss += $sb;

				if ($sb) {

					$this->active_writebuffers[$key][0]->send_buffer = substr($wb[0]->send_buffer, $sb);
					$this->active_writebuffers[$key][0]->buf_size -= $sb;

					if ($this->active_writebuffers[$key][0]->buf_size == 0) {

						$o =& $wb[0]->callback_object;
						$m = $wb[0]->callback_method;
						if (is_callable(array($o, $m))) $o->$m(true);

						array_shift($this->active_writebuffers[$key]);
						if (!count($this->active_writebuffers[$key])) unset($this->active_writebuffers[$key]);

						$sb = false;

					}

				} else {

					$se = $wb[0]->socket->last_error();

					switch ($se) {

						case SOCKET_EWOULDBLOCK:
						$this->active_writebuffers[$key][0]->blocked=true;
						break;

						default:
						if (is_callable(array($o, $m))) $o->$m(false);

						array_shift($this->active_writebuffers[$key]);

						if (!count($this->active_writebuffers[$key])) {
							
							unset($this->active_writebuffers[$key]);

						} else {

							$sb=true;
						
						}

						break;
					
					}
					
				}

			} while ($sb && ($wss<$this->options["max_atom_send"]));

		}
	
	}
	
	function Empty_Buffers() {

		while ($this->active_writebuffers) $this->Run(-1);
	
	}
	
	function SK_Info() {

		return(SK_INFO);
	
	}

}

class SK_Connection_Listener {

	var $pap;
	var $socket;
	var $connection_handler;
	var $options;

	function SK_Connection_Listener($pap, &$socket, $connection_handler, $options=false) {

		$this->pap=$pap;
		$this->socket =& $socket;
		$this->connection_handler=$connection_handler;
		$this->options = $options;
	
	}

}

class SK_Connection_Handler {

	var $conn_id;
	var $server;
	var $socket;
	
	function on_connect() {

		// Method on_connect() is called after an outgoing connection is established
	
	}
	
	function on_accept() { 
	
		// Method on_accept() is called after a client connected to the server
	
	}

	function on_disconnect() {

		// Method on_disconnect() is called after a client disconnected from the server
	
	}

	function on_read($data) {

		// Method on_disconnect() is called after data has been read from the client
	
	}
	
	function disconnect() {

		$this->socket->close();
		unset($this->server->active_sessions[$this->conn_id]);
	
	}

}

class SK_Line_Input_Connection_Handler extends SK_Connection_Handler {

	var $rcv_buffer;
	
	function on_read($data) {

		$this->rcv_buffer.=$data;

		if ($p=strrpos($this->rcv_buffer, "\n")) {

			$lns_arr=explode("\n", substr($this->rcv_buffer, 0, $p-1));
			$this->rcv_buffer=substr($this->rcv_buffer, $p+1);

			foreach ($lns_arr as $line) if ($s=trim($line)) $this->on_read_line($s);
		
		}

	}
	
	function on_read_line($data) {

		// Method on_read_line() is called for each line (\n terminated string) read from the client
	
	}
	
}

class SK_Buffered_Write {

	var $socket;
	var $send_buffer;
	var $callback_object;
	var $callback_method;

	var $buf_size;

	var $blocked=false;
	
	function SK_Buffered_Write($socket, $send_buffer, $callback_object=false, $callback_method=false) {

		$this->socket=$socket;
		$this->send_buffer=$send_buffer;
		$this->callback_object=$callback_object;
		$this->callback_method=$callback_method;

		$this->buf_size=strlen($send_buffer);

	}

}

class SK_Toolkit {

	function techo($str, $filename=false) {

		$tl=date("Ymd:His")." $str\n";

		if (($filename) && ($f=@fopen($filename, "a"))) {

			fwrite($f, $tl);
			fclose($f);
		
		} else {

			echo $tl;
			flush();
		
		}
	
	}
	
	function Run_In_Background($err_code=0) {

		if (is_callable("posix_setsid") && is_callable("pcntl_fork")) {
		
			if (($pid = pcntl_fork()) === 0) {
				
				posix_setsid();
				return true;

			} else {

				exit($err_code);
			
			}
		
		} else {

			return false;
		
		}

	}

}

?>