#!/usr/local/bin/php -q
<?

/*

sh4re - a secure peer to peer file sharing system
=================================================

Copyright (C) 2003 Vincent Negrier aka. sIX <six@aegis-corp.org>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

*/

define("SH4_DEFAULT_CONF_FILE", ((strpos(strtoupper(PHP_OS), "WIN")!==false)?"C:\\sh4re\\":"/etc/sh4re/")."sh4re.conf");
define("SH4_INC_DIR", ((strpos(strtoupper(PHP_OS), "WIN")!==false)?"C:\\sh4re\\inc":"/usr/lib/sh4re/inc"));

include SH4_INC_DIR.DIRECTORY_SEPARATOR."sh4re-core.php";

// GetID3 includes

define("SH4_GETID3_VERSION", "getid3-1.6.4");

include SH4_INC_DIR.DIRECTORY_SEPARATOR.SH4_GETID3_VERSION.DIRECTORY_SEPARATOR."getid3.php";

// File catalog funcs

function read_metadata($filename) {

	$ret = array();
	$meta = getallfileinfo($filename);

	if (is_array($meta["comments"])) foreach ($meta["comments"] as $k=>$cmt) $ret[ucfirst($k)] = (is_array($cmt)?$cmt[0]:$cmt);

	// Clean some mess

	unset($ret["Genreid"]);

	// Continue

	$has_vid = is_array($meta["video"]);
	$has_aud = is_array($meta["audio"]);

	if ($has_vid) $ret["file_type"] = "Video";
	else if ($has_aud) $ret["file_type"] = "Audio";
	else $ret["file_type"] = "Other";
	
	if ($v = $meta["playtime_string"]) $ret["Play time"] = $v;
	
	if ($v = $meta["video"]["codec"]) $ret["Video codec"] = $v;
	if ($v = $meta["video"]["resolution_x"]) $ret["Video resolution"] = $v."x".$meta["video"]["resolution_y"];
	if ($v = $meta["video"]["bitrate"]) $ret["Video bitrate"] = (int)($v/1024)." Kbit/s";
	if ($v = $meta["video"]["frame_rate"]) $ret["Video framerate"] = $v." fps";
	if ($v = $meta["video"]["bitrate_mode"]) $ret["Video BR mode"] = $v;

	if ($v = $meta["audio"]["codec"]) $ret["Audio codec"] = $v;
	if ($v = $meta["audio"]["channels"]) $ret["Audio channels"] = $v;
	if ($v = $meta["audio"]["bitrate"]) $ret["Audio bitrate"] = (int)($v/1024)." Kbit/s";
	if ($v = $meta["audio"]["codec"]) $ret["Audio codec"] = $v;
	if ($v = $meta["audio"]["sample_rate"]) $ret["Audio rate"] = $v." Hz";
	if ($v = $meta["audio"]["bitrate_mode"]) $ret["Audio BR mode"] = $v;

	return $ret;

}

function read_directory($dirname) {

	static $n_items = 0;
	static $n_bytes = 0;
	
	$t_files = array();

	$d = opendir($dirname);
	while (($f = readdir($d)) !== false) $t_files[] = $f;
	closedir($d);

	// Pass 1 : search for deleted items

	$h_cache = array();
	
	if ($db_dir = sh4_get_file_ids_by_dir($dirname)) foreach ($db_dir as $db_entry_id) {

		$db_f = sh4_get_file($db_entry_id, false);
		$db_fname = $db_f["file_name"];

		if (!in_array($db_fname, $t_files)) {

			// File has disapeared !

			sh4_kill_file($db_entry_id);

		} 

	}

	// Pass 2 : read files

	foreach ($t_files as $f) {
	
		$cfn = $dirname.DIRECTORY_SEPARATOR.$f;

		if (!is_readable($cfn)) continue;

		if (is_dir($cfn)) {

			if ($f=="." || $f=="..") continue;
			if (in_array($cfn, $GLOBALS["conf"]["repositoryexclude"])) continue;

			read_directory($cfn);
		
		} else {

			$item = array();
			
			$item["file_directory"] = $dirname;
			$item["file_parent_dir"] = substr($dirname, strrpos($dirname, DIRECTORY_SEPARATOR)+1);
			$item["file_name"] = $f;
			$item["file_date"] = date("Y-m-d H:i:s", filemtime($cfn));
			$item["file_size"] = filesize($dirname.DIRECTORY_SEPARATOR.$f);

			if ($this_f_id = sh4_get_file_id_by_DN($item["file_directory"], $item["file_name"])) {
	
				$this_f = sh4_get_file($this_f_id, false);

				if (($this_f["file_date"] != $item["file_date"]) || ($this_f["file_size"] != $item["file_size"])) {
					
					sh4_kill_file($this_f_id);
					unset($this_f_id);

				}
			
			}
			
			if (!$this_f_id) {
			
				$store_f = true;
				$item["file_hash"] = sh4_compute_hash($cfn);
			
				if ($dupe_id = sh4_get_file_id_by_hash($item["file_hash"])) {
					
					$dupe_f = sh4_get_file($dupe_id, false);
					$dupe_cfn = $dupe_f["file_directory"].DIRECTORY_SEPARATOR.$dupe_f["file_name"];

					if (!file_exists($dupe_cfn)) {

						// File had been moved

						$store_f = true;
						sh4_kill_file($dupe_id);

					} else {
					
						// This is a real dupe
						
						$store_f = false;
						if ($item["file_size"]>16) $GLOBALS["dupes"][$dupe_id][] = $dirname.DIRECTORY_SEPARATOR.$f;

					}
				
				} 

				if ($store_f) {
					
					// Store file data

					if ($meta = read_metadata($cfn)) foreach ($meta as $k=>$v) $item[$k] = $v;
					sh4_put_file($item);

				}
			
			}
			
			$n_bytes += $item["file_size"];
			++$n_items;	
	
		}

	}

	echo_scan($n_items, $n_bytes, $dirname);

}

// Output funcs

function echo_header() {

	echo "sh4re-update/".SH4_VERSION." (c) 2003 by sIX / aEGiS <six@t0x.net>";
	echo "\n\n";

}

function echo_connect() {

	echo "connecting to MySQL       ... ";

}

function echo_ok() {

	echo "OK\n";

}

function echo_fail() {

	echo "FAILED\n";

}

function echo_scan_begin() {

	echo "scanning repository       ... ";

}

function echo_scan($n_items, $n_bytes, $dirname) {

	echo chr(13)."scanning repository       ... (".sprintf("%.1f", $n_bytes/1073741824)."GB in ".(int)$n_items." items)";

}

function echo_scan_end() {

	echo chr(13)."                                                                             ";
	echo chr(13)."scanning repository       ... ";

}

function echo_help() {

	echo $GLOBALS["argv"][0]." [options]\n\n";
	echo "options : -h --help       : show this help page\n";
	echo "          -V --version    : show version\n";
	echo "          -r --rebuild    : rebuild whole database\n";
	echo "          -c --config=xx  : use sepcified config file\n";
	echo "\n";

}

function echo_dupelog() {

	echo "writing dupes log         ... ";

}

function echo_done($elapsed) {

	$m = (int)($elapsed/60);
	$s = $elapsed%60;
	
	echo "\n";
	echo "update completed in ".($m?($m." minute".(($m>1)?"s":"").", "):"").((int)$s." second".(($s>1)?"s":""))."\n\n";

}

// Other funcs

// Begin

set_time_limit(0);

$start_time = time();

$cmdline_args = sh4_parse_cmdline_args(array (	"-h" => "--help",
												"-V" => "--version",
												"-r" => "--rebuild",
												"-c" => "--config"));

if ($cmdline_args["-V"]) die(VERSION."\n");

echo_header();

if (version_compare(phpversion(), "4.3.0")<0) die("sh4re-update needs PHP >= 4.3)");

// Init

if ($cmdline_args["-h"]) {

	echo_help();
	die();

}

$conf_f = $cmdline["-c"]
or $conf_f = SH4_DEFAULT_CONF_FILE;

if (!$conf = sh4_parse_config($conf_f)) errexit("FATAL: unable to read conf file '$conf_f'");

foreach ($conf["getid3module"] as $id3mod) include SH4_INC_DIR.DIRECTORY_SEPARATOR.SH4_GETID3_VERSION.DIRECTORY_SEPARATOR."getid3.".$id3mod.".php";

echo_connect();

if ((!mysql_connect($conf["mysqlhost"][0], $conf["mysqluser"][0], $conf["mysqlpass"][0])) || (!mysql_select_db($conf["mysqldb"][0]))) {

	echo_fail();
	die();

} else {

	echo_ok();

}

$q = mysql_query("desc files");
while ($r = mysql_fetch_row($q)) $main_cols[] = $r[0];
mysql_free_result($q);

// Clean if asked

if ($cmdline_args["-r"]) {

	mysql_query("truncate table files");
	mysql_query("truncate table files_metadata");

}

// Update catalog

$dupes = array();

echo_scan_begin();
foreach ($conf["repository"] as $cr) read_directory($cr);
echo_scan_end();
echo_ok();

// Write dupelog

if ($dupes && $conf["dupeslog"][0]) {

	echo_dupelog();
	
	$f = fopen($conf["dupeslog"][0], "a");
	fputs($f, "sh4re-update/".SH4_VERSION." dupes on ".date("Y-m-d H:i:s")." : \n");

	foreach ($dupes as $fid=>$dnames) {

		$file = sh4_get_file($fid, false);
		fputs($f, "---\n".$file["file_directory"].DIRECTORY_SEPARATOR.$file["file_name"]."\n");
		foreach ($dnames as $dname) fputs($f, $dname."\n");
	
	}

	fputs($f, "---\n\n");
	fclose($f);

	echo_ok();

}

unset($dupes);

if ($pid = trim(@file_get_contents($conf["pidfile"][0]))) {

	// Ask sh4red to reload from database
	
	posix_kill($pid, SIGUSR1);

}

$elapsed = time() - $start_time;

echo_done($elapsed);

?>